package com.experimental.irrigator

import android.bluetooth.BluetoothSocket
import android.os.Bundle
import android.os.Handler
import android.util.Log
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers.IO
import kotlinx.coroutines.launch
import java.io.IOException
import java.io.InputStream
import java.io.OutputStream
import kotlin.coroutines.CoroutineContext

private const val TAG = "-- BlueService"

// Defines several constants used when transmitting messages between the
// service and the UI.
const val MESSAGE_READ: Int = 0
const val MESSAGE_WRITE: Int = 1
const val MESSAGE_TOAST: Int = 2
const val MESSAGE_READY: Int = 3
const val MESSAGE_CLOSED: Int = 4
// ... (Add other message types here as needed.)


class BlueService(  // handler that gets info from Bluetooth service
                    private val handler: Handler ) {

    private var mConnectedThread : ConnectedThread? = null

    private inner class ConnectedThread(private val mmSocket: BluetoothSocket) : Thread() {

        private val mmInStream: InputStream = mmSocket.inputStream
        private val mmOutStream: OutputStream = mmSocket.outputStream
        private val mmBuffer: ByteArray = ByteArray(1024) // mmBuffer store for the stream

        override fun run() {
            var numBytes: Int // bytes returned from read()

            Log.d( TAG, "signaling connection thread readyness")
            val readMsg = handler.obtainMessage( MESSAGE_READY )
            readMsg.sendToTarget()
            // Keep listening to the InputStream until an exception occurs.
            while (true) {
                // Read from the InputStream.
                numBytes = try {
                    mmInStream.read(mmBuffer)
                } catch (e: IOException) {
                    Log.d(TAG, "Input stream was disconnected", e)
                    cancel()
                    Log.d( TAG, "signaling closed communications thread")
                    val readMsg = handler.obtainMessage( MESSAGE_CLOSED )
                    readMsg.sendToTarget()
                    break
                }

                // Send the obtained bytes to the UI activity.
                Log.d(TAG, "received bytes: " +numBytes)
                val readMsg = handler.obtainMessage(
                    MESSAGE_READ, numBytes, -1,
                    mmBuffer)
                readMsg.sendToTarget()
            }
        }

        // Call this from the main activity to send data to the remote device.
        fun write(bytes: ByteArray) {
            try {
                mmOutStream.write(bytes)
            } catch (e: IOException) {
                Log.e(TAG, "Error occurred when sending data", e)

                // Send a failure message back to the activity.
                val writeErrorMsg = handler.obtainMessage(MESSAGE_TOAST)
                val bundle = Bundle().apply {
                    putString("toast", "Couldn't send data to the other device")
                }
                writeErrorMsg.data = bundle
                handler.sendMessage(writeErrorMsg)
                return
            }

            // Share the sent message with the UI activity.
            val writtenMsg = handler.obtainMessage(
                MESSAGE_WRITE, -1, -1, bytes)
            writtenMsg.sendToTarget()
        }

        // Call this method from the main activity to shut down the connection.
        fun cancel() {
            try {
                mmSocket.close()
            } catch (e: IOException) {
                Log.e(TAG, "Could not close the connect socket", e)
            }
        }
    }
    public fun communicate( socket : BluetoothSocket ) {
        mConnectedThread?.cancel()
        mConnectedThread = ConnectedThread(socket)
        Log.d(TAG, "starting comms thread")
        //CoroutineScope(IO).launch {
            mConnectedThread?.run()
        //}
    }

    public fun sendByte( byte : Byte ) {
        mConnectedThread?.write(byteArrayOf(byte))
    }

    public fun cancel() {
        mConnectedThread?.cancel()
    }
}
