#include <Arduino.h>
#include "BluetoothSerial.h"

#define DEBUG 1
#define LED 2
#define SHORTBLINK 100

#define STATREQUEST 0xa1
#define FILLRESET   0xa2

#if !defined(CONFIG_BT_ENABLED) || !defined(CONFIG_BLUEDROID_ENABLED)
#error Bluetooth is not enabled! Please run `make menuconfig` to and enable it
#endif

BluetoothSerial SerialBT;

void blinkLED(uint16_t milLenght);
uint8_t getMoisture(void);
uint8_t getFill(void);
uint8_t getTemp(void);

void setup() {
  // put your setup code here, to run once:
  if (DEBUG) Serial.begin(115200);
  SerialBT.begin("IPAT0.1"); //Bluetooth device name
  if (DEBUG) Serial.println("The device started, now you can pair it with bluetooth!");
  pinMode(LED,OUTPUT);
  for (short i = 0; i<2; i++) blinkLED(SHORTBLINK);
}

void loop() {
  // put your main code here, to run repeatedly:
  // Perform control tasks?
  if (DEBUG) Serial.println("\nPerforming control tasks ..");
  // .. state-machine? ..
  // ..
  // Check Bluetooth (response timeout after 200ms by app)
  if (DEBUG) Serial.println("\nCheckling/Waiting for connections ..");
  if (!SerialBT.connected(1000)) { // Wait a second for connections
    if (DEBUG) Serial.println("\nno connections ..");
  } else if (DEBUG) Serial.println("\nconnected ..");
  if(SerialBT.available()) {
    int msg = SerialBT.read();
    if (DEBUG) {
      Serial.print("\nreceived msg "); Serial.println(msg,HEX);
    }
    if ( msg == STATREQUEST ) {
      // Send stats
      uint8_t msg[] = { getMoisture(), getFill(), getTemp() };
      SerialBT.write(msg, 3);
      if (DEBUG) blinkLED(SHORTBLINK);
    } else if ( msg == FILLRESET ) {
      // Send confirmation and reset fill
      SerialBT.write(FILLRESET);
      if (DEBUG) blinkLED(SHORTBLINK);
      if (DEBUG) Serial.println("\nresetting fill ..");
      // ..
      // ..
    } else { if (DEBUG) { Serial.println("\nWARNING unrecognized msg"); } }
  }
}

void blinkLED(uint16_t milLenght) {
  pinMode(LED, HIGH);
  delay(milLenght);
  pinMode(LED, LOW);
  delay(milLenght);
}
uint8_t getMoisture() {
  // dummy value
  return (char) (75 -(millis()/1000)%50);
}
uint8_t getFill() {
  // dummy value
  return (char) (100 -(millis()/1000)%75);
}
uint8_t getTemp() {
  // dummy value
  return (char) 21;
}
