#include <Arduino.h>
#include <OneWire.h>
#include <DallasTemperature.h>
#include <Preferences.h>
#include "BluetoothSerial.h"


#define STATREQUEST 0xa1
#define FILLRESET   0xa2

#if !defined(CONFIG_BT_ENABLED) || !defined(CONFIG_BLUEDROID_ENABLED)
#error Bluetooth is not enabled! Please run `make menuconfig` to and enable it
#endif

BluetoothSerial SerialBT;

// Data wire is plugged into pin 19 on the Arduino (Temperatursensor)
#define ONE_WIRE_BUS 19

//Ultrasonic sensor pins
#define TRIGGER 23
#define ECHO 34

//Humidity sensor pin
#define HUMIDITY 33

//LED PINS
#define RED 27
#define GREEN 26
#define BLUE 25

//Pumpe
#define PUMPE 32

// setting PWM properties
const int freq = 5000;
const int ledChannelRED = 0;
const int ledChannelGREEN = 1;
const int ledChannelBLUE = 2;
const int resolution = 8;

// Setup a OneWire instance to communicate with any OneWire devices
OneWire oneWire(ONE_WIRE_BUS);

// Pass OneWire reference to Dallas Temperature
DallasTemperature sensors(&oneWire);

//Variablen fur Ultraschallmessung
long dauer=0;
long entfernung=0;

//Variablen fuer Ablaufsteuerung
int state = 0;  //0:Start, 1:wait, 2:water, 3:tank empty, 4:Error
int humidity;
int temperature;
int hour;
int minute;
int waterlevel;
long distance_floor;
//
Preferences preferences;

//Sensorwerte fuer App
uint8_t Moisture = 0;
uint8_t Fill = 0;
uint8_t Temp = 0;
//timer 
hw_timer_t * timer = NULL;

//flag für bewaesserung
bool done = true;

void IRAM_ATTR onTime(){
  digitalWrite(PUMPE, LOW);
  ledcWrite(ledChannelGREEN, 0);
  timerAlarmDisable(timer);
  done = true;
}

void water(int duration){
  ledcWrite(ledChannelGREEN, 255);
  digitalWrite(PUMPE, HIGH);
  timerAlarmWrite(timer, 10000000, true);			
	timerAlarmEnable(timer);
}
void get_distance_floor(){
  //measure distance
  digitalWrite(TRIGGER, LOW);
  delayMicroseconds(2);
  digitalWrite(TRIGGER, HIGH);
  delayMicroseconds(10);
  digitalWrite(TRIGGER, LOW);
  dauer = pulseIn(ECHO, HIGH);
  distance_floor = (dauer/2)/29.1;

  //write distance to Flash
  preferences.begin("Data", false);
  preferences.putLong("distance", distance_floor);
  preferences.end();
}

void setup() { 
  Serial.begin(115200);

  SerialBT.begin("IPAT0.1"); //Bluetooth device name
  Serial.println("The device started, now you can pair it with bluetooth!");
  //initialize temperature sensor
  sensors.begin();

  pinMode(TRIGGER, OUTPUT);
  pinMode(ECHO, INPUT);
  pinMode(PUMPE, OUTPUT);

  pinMode(HUMIDITY, INPUT);

  //configure LED PWM functionalities and attach the channel to GPIOs
  ledcSetup(ledChannelRED, freq, resolution);
  ledcSetup(ledChannelGREEN, freq, resolution);
  ledcSetup(ledChannelBLUE, freq, resolution);
  ledcAttachPin(RED, ledChannelRED);
  ledcAttachPin(GREEN, ledChannelGREEN);
  ledcAttachPin(BLUE, ledChannelBLUE);

  //get distance to floor from Flash
  preferences.begin("Data", false);
  distance_floor = preferences.getLong("distance", 0);
  preferences.end();

  //configure timer
  timer = timerBegin(0, 80, true);                
	timerAttachInterrupt(timer, &onTime, true); 
  
  
}

void loop() {
  Serial.print("distance floor: ");
  Serial.println(distance_floor);
  Serial.print("humidity: ");
  Serial.println(Moisture);
  Serial.print("temperatur: ");
  Serial.println(Temp);
  Serial.print("Füllstand: ");
  Serial.println(Fill);
   // Perform control tasks?
  Serial.println("\nPerforming control tasks ..");

  // Check Bluetooth (response timeout after 200ms by app)
  Serial.println("\nCheckling/Waiting for connections ..");
  if (!SerialBT.connected(1000)) { // Wait a second for connections
    Serial.println("\nno connections ..");
  } 
  else Serial.println("\nconnected ..");
  if(SerialBT.available()) {
    int msg = SerialBT.read();
    Serial.print("\nreceived msg "); 
    Serial.println(msg,HEX);
    if ( msg == STATREQUEST ) {
      // Send stats
      uint8_t msg[] = { Moisture, Fill, Temp };
      SerialBT.write(msg, 3);
    } else if ( msg == FILLRESET ) {
      // Send confirmation and reset fill
      SerialBT.write(FILLRESET);
      Serial.println("\nresetting fill ..");
      get_distance_floor();
      
    } else Serial.println("\nWARNING unrecognized msg");
    while (SerialBT.available()) {
      Serial.println("WARNING unexpected data");
      SerialBT.read();
    }
  }
  //read waterlevel
  digitalWrite(TRIGGER, LOW);
  delayMicroseconds(2);
  digitalWrite(TRIGGER, HIGH);
  delayMicroseconds(10);
  digitalWrite(TRIGGER, LOW);
  dauer = pulseIn(ECHO, HIGH);
  float distance = (dauer/2)/29.1;
  waterlevel = ((float)distance_floor - distance)/(float)distance_floor*100;
  if (waterlevel >= 0){
    Fill = (uint8_t)waterlevel;
  }
  else Fill = 0;
  Serial.print("waterlevel: ");
  Serial.println(waterlevel);
  //read temperature
  sensors.requestTemperatures();
  temperature = sensors.getTempCByIndex(0);
  Temp = (uint8_t)temperature;
  //read humidity
  int humidity_value = analogRead(HUMIDITY);
  Serial.print("Feuchte wert: ");
  Serial.println(humidity_value);
  //1050 ^= 100% ; 2700^=0%
  humidity = 100. - ((humidity_value - 1050.) / (2700. - 1050.) * 100.);
  if (humidity >= 0){
    Moisture = (uint8_t)humidity;  
  }
  else Moisture = 0;
  Serial.print("Feuchtigkeit: ");
  Serial.println(humidity);

  switch (state) {
    case 0:   //Start
      state = 1;
      break;
    case 1:   //wait 
      {
        
        //niedrigen Wasserlevel anzeigen
        if (waterlevel < 10){
          ledcWrite(ledChannelBLUE, 255);
        }
        else{
          ledcWrite(ledChannelBLUE, 0);
        }
        
        //check for sensor Errors
        if (humidity < 0 or humidity > 100 or dauer == 0 or temperature < -55 or distance == 0){
          state = 4;
        } 
        else if (waterlevel <= 2){
          state = 3;
        }
        else if (humidity < 30 and temperature > 4) {
          state = 2;
        }
        //in sleepmode??
        break;
      }
    case 2:
        if (done){
          water(10);
          done = false;
        }
        state = 1;
        break;

    case 3:   //tank empty
      ledcWrite(ledChannelRED, 255);
      if (waterlevel > 5){
        state = 1;
        ledcWrite(ledChannelRED, 0);
      }
      break;
    case 4:   
      ledcWrite(ledChannelRED, 255);
      ledcWrite(ledChannelBLUE, 0);
      ledcWrite(ledChannelGREEN, 0);
      if (not (humidity < 0 or humidity > 100 or dauer == 0 or temperature < -55 or distance == 0)){
        state = 1;
        ledcWrite(ledChannelRED, 0);
      }
      
      break;
    
  }
}